local VERSION = tonumber(("$Revision: 44600 $"):match("%d+"))

local Parrot = Parrot
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-23 05:02:57 -0400 (Mon, 23 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local mod = Parrot:NewModule("CombatEvents_Data", "AceEvent-2.0", "Parser-3.0")

local BS = AceLibrary("Babble-Spell-2.2")
local Gratuity -- AceLibrary("Gratuity-2.0")

local L = Parrot:L("Parrot_CombatEvents_Data", {
	["Miss!"] = MISS .. "!",
	["Dodge!"] = DODGE .. "!",
	["Parry!"] = PARRY .. "!",
	["Block!"] = BLOCK .. "!",
	["Absorb!"] = ABSORB .. "!",
	["Immune!"] = IMMUNE .. "!",
	["Resist!"] = RESIST .. "!",
	["Reflect!"] = REFLECT .. "!",
	["Interrupt!"] = INTERRUPT .. "!",
	["Pet Miss!"] = PET .. " " .. MISS .. "!",
	["Pet Dodge!"] = PET .. " " .. DODGE .. "!",
	["Pet Parry!"] = PET .. " " .. PARRY .. "!",
	["Pet Block!"] = PET .. " " .. BLOCK .. "!",
	["Pet Absorb!"] = PET .. " " .. ABSORB .. "!",
	["Pet Immune!"] = PET .. " " .. IMMUNE .. "!",
	["Pet Resist!"] = PET .. " " .. RESIST .. "!",
	["Pet Reflect!"] = PET .. " " .. REFLECT .. "!",
	["Pet Interrupt!"] = PET .. " " .. INTERRUPT .. "!",
})

local new, del = Parrot.new, Parrot.del

local onEnableFuncs = {}
function mod:OnEnable()
	assert(AceLibrary("SpecialEvents-Aura-2.0")) -- force a load
	Gratuity = AceLibrary("Gratuity-2.0")
	
	for _,v in ipairs(onEnableFuncs) do
		v()
	end
end

------------------------------------------------------------------------------
-- Incoming events -----------------------------------------------------------
------------------------------------------------------------------------------
local coloredDamageAmount = function(info)
	local db = Parrot:GetModule("CombatEvents").db.profile.damageTypes
	if db.color then
		return "|cff" .. db[info.damageType] .. info.amount .. "|r"
	else
		return info.amount
	end
end

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee damage",
	localName = L["Melee damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
	},
	color = "ff0000", -- red
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee misses",
	localName = L["Melee misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee dodges",
	localName = L["Melee dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee parries",
	localName = L["Melee parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee blocks",
	localName = L["Melee blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee absorbs",
	localName = L["Melee absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee immunes",
	localName = L["Melee immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "ffff00", -- yellow
}

local function retrieveAbilityName(info)
	return Parrot:GetModule("CombatEvents"):GetAbbreviatedSpell(info.abilityName)
end

local function retrieveIconFromAbilityName(info)
	local spell = info.abilityName
	if BS:HasReverseTranslation(spell) then
		return BS:GetSpellIcon(BS.reverse[spell])
	end
end

local function retrieveIconFrom1(info)
	local spell = info[1]
	if BS:HasReverseTranslation(spell) then
		return BS:GetSpellIcon(BS.reverse[spell])
	end
end

local function retrieveIconFromItemLink(info)
	local itemLink = info.itemLink
	if itemLink then
		local _, _, _, _, _, _, _, _, _, texture = GetItemInfo(itemLink)
		return texture
	end
end

Parrot:RegisterThrottleType("Skill damage", L["Skill damage"], 0.1, true)

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill damage",
	localName = L["Skill damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that the enemy attacked you with."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ff0000", -- red
	canCrit = true,
	throttle = { "Skill damage", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
}

Parrot:RegisterThrottleType("DoTs and HoTs", L["DoTs and HoTs"], 2)

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that the enemy attacked you with."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ff0000", -- red
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability misses",
	localName = L["Ability misses"],
	defaultTag = "([Skill]) " .. L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability dodges",
	localName = L["Ability dodges"],
	defaultTag = "([Skill]) " .. L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability parries",
	localName = L["Ability parries"],
	defaultTag = "([Skill]) " .. L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability blocks",
	localName = L["Ability blocks"],
	defaultTag = "([Skill]) " .. L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell resists",
	localName = L["Spell resists"],
	defaultTag = "([Skill]) " .. L["Resist!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "7f007f", -- purple
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill absorbs",
	localName = L["Skill absorbs"],
	defaultTag = "([Skill]) " .. L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill immunes",
	localName = L["Skill immunes"],
	defaultTag = "([Skill]) " .. L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill reflects",
	localName = L["Skill reflects"],
	defaultTag = "([Skill]) " .. L["Reflect!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "7f007f", -- purple
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell interrupts",
	localName = L["Skill interrupts"],
	defaultTag = "([Skill]) " .. L["Interrupt!"],
	parserEvent = {
		eventType = "Interrupt",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterFilterType("Incoming heals", L["Incoming heals"], 0)

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally that healed you."],
		Skill = L["The spell or ability that the ally healed you with."],
		Amount = L["The amount of healing done."],
	},
	color = "00ff00", -- green
	canCrit = true,
	filterType = { "Incoming heals", 'realAmount' },
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals over time",
	localName = L["Heals over time"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally that healed you."],
		Skill = L["The spell or ability that the ally healed you with."],
		Amount = L["The amount of healing done."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
	color = "00ff00", -- green
	filterType = { "Incoming heals", 'realAmount' },
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Environmental damage",
	localName = L["Environmental damage"],
	defaultTag = "-[Amount] [Type]",
	parserEvent = {
		eventType = "Environmental",
		recipientID = "player",
	},
	tagTranslations = {
		Amount = coloredDamageAmount,
		Type = "hazardTypeLocal",
	},
	tagTranslationsHelp = {
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
	},
	color = "ff0000", -- red
}

------------------------------------------------------------------------------
-- Outgoing events -----------------------------------------------------------
------------------------------------------------------------------------------
Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee damage",
	localName = L["Melee damage"],
	defaultTag = "[Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
	},
	color = "ffffff", -- white
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee misses",
	localName = L["Melee misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee dodges",
	localName = L["Melee dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee parries",
	localName = L["Melee parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee blocks",
	localName = L["Melee blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee absorbs",
	localName = L["Melee absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee immunes",
	localName = L["Melee immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee evades",
	localName = L["Melee evades"],
	defaultTag = L["Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill damage",
	localName = L["Skill damage"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that you used."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	canCrit = true,
	throttle = { "Skill damage", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that you used."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability misses",
	localName = L["Ability misses"],
	defaultTag = L["Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability dodges",
	localName = L["Ability dodges"],
	defaultTag = L["Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability parries",
	localName = L["Ability parries"],
	defaultTag = L["Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}


Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability blocks",
	localName = L["Ability blocks"],
	defaultTag = L["Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell resists",
	localName = L["Spell resists"],
	defaultTag = L["Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill absorbs",
	localName = L["Skill absorbs"],
	defaultTag = L["Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill immunes",
	localName = L["Skill immunes"],
	defaultTag = L["Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill reflects",
	localName = L["Skill reflects"],
	defaultTag = L["Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell interrupts",
	localName = L["Skill interrupts"],
	defaultTag = L["Interrupt!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Interrupt",
		sourceID = "player",
		recipientID_not = "player",
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}


Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill evades",
	localName = L["Skill evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterFilterType("Outgoing heals", L["Outgoing heals"], 0)

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally you healed."],
		Skill = L["The spell or ability that you used."],
		Amount = L["The amount of healing done."],
	},
	color = "00ff00", -- green
	canCrit = true,
	filterType = { "Outgoing heals", 'realAmount' },
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals over time",
	localName = L["Heals over time"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally you healed."],
		Skill = L["The spell or ability that you used."],
		Amount = L["The amount of healing done."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
	color = "00ff00", -- green
	filterType = { "Outgoing heals", 'realAmount' },
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee damage",
	localName = L["Pet melee damage"],
	defaultTag = "Pet [Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Amount = L["The amount of damage done."],
	},
	color = "ff7f00", -- orange
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee misses",
	localName = L["Pet melee misses"],
	defaultTag = L["Pet Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee dodges",
	localName = L["Pet melee dodges"],
	defaultTag = L["Pet Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee parries",
	localName = L["Pet melee parries"],
	defaultTag = L["Pet Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee blocks",
	localName = L["Pet melee blocks"],
	defaultTag = L["Pet Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee absorbs",
	localName = L["Pet melee absorbs"],
	defaultTag = L["Pet Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "7f7fff", -- light blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee immunes",
	localName = L["Pet melee immunes"],
	defaultTag = L["Pet Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "7f7fff", -- light blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee evades",
	localName = L["Pet melee evades"],
	defaultTag = L["Pet Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7fff", -- pink
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill damage",
	localName = L["Pet skill damage"],
	defaultTag = "Pet [Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability misses",
	localName = L["Pet ability misses"],
	defaultTag = L["Pet Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability dodges",
	localName = L["Pet ability dodges"],
	defaultTag = L["Pet Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability parries",
	localName = L["Pet ability parries"],
	defaultTag = L["Pet Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability blocks",
	localName = L["Pet ability blocks"],
	defaultTag = L["Pet Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet spell resists",
	localName = L["Pet spell resists"],
	defaultTag = L["Pet Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill absorbs",
	localName = L["Pet skill"],
	defaultTag = L["Pet Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill immunes",
	localName = L["Pet skill immunes"],
	defaultTag = L["Pet Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill reflects",
	localName = L["Pet skill reflects"],
	defaultTag = L["Pet Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill evades",
	localName = L["Pet skill evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "ff7fff", -- pink
}

------------------------------------------------------------------------------
-- Notification events -------------------------------------------------------
------------------------------------------------------------------------------

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Debuff gains",
	localName = L["Debuff gains"],
	defaultTag = "([Name])",
	blizzardEvent = "SpecialEvents_PlayerDebuffGained",
	tagTranslations = {
		Name = 1,
		Icon = 4,
	},
	tagTranslationsHelp = {
		Name = L["The name of the debuff gained."],
	},
	color = "007f7f", -- dark cyan
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Buff gains",
	localName = L["Buff gains"],
	defaultTag = "([Name])",
	blizzardEvent = "SpecialEvents_PlayerBuffGained",
	tagTranslations = {
		Name = 1,
		Icon = 4,
	},
	tagTranslationsHelp = {
		Name = L["The name of the buff gained."],
	},
	color = "b2b200", -- dark yellow
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Item buff gains",
	localName = L["Item buff gains"],
	defaultTag = "([Name])",
	tagTranslations = {
		Name = 1,
		Icon = retrieveIconFrom1,
	},
	tagTranslationsHelp = {
		Name = L["The name of the item buff gained."],
	},
	color = "b2b2b2", -- gray
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Debuff fades",
	localName = L["Debuff fades"],
	defaultTag = "-([Name])",
	blizzardEvent = "SpecialEvents_PlayerDebuffLost",
	tagTranslations = {
		Name = 1,
		Icon = 4,
	},
	tagTranslationsHelp = {
		Name = L["The name of the debuff lost."],
	},
	color = "00d8d8", -- cyan
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Buff fades",
	localName = L["Buff fades"],
	defaultTag = "-([Name])",
	blizzardEvent = "SpecialEvents_PlayerBuffLost",
	tagTranslations = {
		Name = 1,
		Icon = 3,
	},
	tagTranslationsHelp = {
		Name = L["The name of the buff lost."],
	},
	color = "e5e500", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Item buff fades",
	localName = L["Item buff fades"],
	defaultTag = "-([Name])",
	tagTranslations = {
		Name = 1,
		Icon = retrieveIconFrom1,
	},
	tagTranslationsHelp = {
		Name = L["The name of the item buff lost."],
	},
	color = "e5e5e5", -- light gray
}

local function GetItemBuffName(slot)
	Gratuity:SetInventoryItem("player", slot)
	
	local _, _, buffName = Gratuity:Find("^([^%(]+) %(%d+ [^%)]+%)$", nil, nil, false, true)
	return buffName
end

local mainhandItemBuff
local offhandItemBuff
local function refreshItemBuffs()
	local mainhand, _, _, offhand = GetWeaponEnchantInfo()
	mainhandItemBuff = mainhand and GetItemBuffName(GetInventorySlotInfo("MainHandSlot"))
	offhandItemBuff = offhand and GetItemBuffName(GetInventorySlotInfo("SecondaryHandSlot"))
end
onEnableFuncs[#onEnableFuncs+1] = function()
	mod:ScheduleRepeatingEvent("Parrot_CombatEvents_Data:ItemBuffCheck", mod.ItemBuffCheck, 0.2, mod)
	
	refreshItemBuffs()
end
function mod:ItemBuffCheck()
	local oldMainhandItemBuff = mainhandItemBuff
	local oldOffhandItemBuff = offhandItemBuff
	refreshItemBuffs()
	if oldMainhandItemBuff ~= mainhandItemBuff then
		if oldMainhandItemBuff then
			-- loss
			local info = new(oldMainhandItemBuff)
			Parrot:TriggerCombatEvent("Notification", "Item buff fades", info)
			info = del(info)
		end
		if mainhandItemBuff then
			-- gain
			local info = new(mainhandItemBuff)
			Parrot:TriggerCombatEvent("Notification", "Item buff gains", info)
			info = del(info)
		end
	end
	if oldOffhandItemBuff ~= offhandItemBuff then
		if oldOffhandItemBuff then
			-- loss
			local info = new(oldOffhandItemBuff)
			Parrot:TriggerCombatEvent("Notification", "Item buff fades", info)
			info = del(info)
		end
		if offhandItemBuff then
			-- gain
			local info = new(offhandItemBuff)
			Parrot:TriggerCombatEvent("Notification", "Item buff gains", info)
			info = del(info)
		end
	end
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Enter combat",
	localName = L["Enter combat"],
	defaultTag = "+Combat",
	blizzardEvent = "PLAYER_REGEN_DISABLED",
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Leave combat",
	localName = L["Leave combat"],
	defaultTag = "-Combat",
	blizzardEvent = "PLAYER_REGEN_ENABLED",
	color = "ffffff", -- white
}

Parrot:RegisterThrottleType("Power gain/loss", L["Power gain/loss"], 3)
Parrot:RegisterFilterType("Power gain", L["Power gain"], 0)

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Power gain",
	localName = L["Power gain"],
	defaultTag = L["+[Amount] [Type]"],
	parserEvent = {
		eventType = "Gain",
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = function(info)
			return info.amountGained or info.amount
		end,
		Type = function(info)
			return info.attributeGainedLocal or info.attributeLocal
		end,
		Skill = retrieveAbilityName,
		Name = function(info)
			return info.sourceGainedName and info.recipientName or info.sourceName
		end,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Amount = L["The amount of power gained."],
		Type = L["The type of power gained (Mana, Rage, Energy)."],
		Skill = L["The ability or spell used to gain power."],
		Name = L["The character that the power comes from."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	throttle = { "Power gain/loss", 'abilityName', 'throttleCount', sourceName = L["Multiple"], recipientName = L["Multiple"] },
	filterType = { "Power gain", function(info)
		return info.amountGained or info.amount
	end}
}

onEnableFuncs[#onEnableFuncs+1] = function()
	mod:RegisterParserEvent({
		eventType = "Leech",
		sourceGainedID = "player",
		attributeGained_in = { "Mana", "Focus", "Rage", "Energy" },
	}, function(info)
		Parrot:TriggerCombatEvent("Notification", "Power gain", info)
	end)
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Power loss",
	localName = L["Power loss"],
	defaultTag = L["-[Amount] [Type]"],
	parserEvent = {
		eventType_in = { "Drain", "Leech" },
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = "amount",
		Type = "attributeLocal",
		Skill = retrieveAbilityName,
		Name = "sourceName",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Amount = L["The amount of power lost."],
		Type = L["The type of power lost (Mana, Rage, Energy)."],
		Skill = L["The ability or spell take away your power."],
		Name = L["The character that caused the power loss."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	throttle = { "Power gain/loss", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Combo point gain",
	localName = L["Combo point gain"],
	defaultTag = L["[Num] CP"],
	tagTranslations = {
		Num = 1
	},
	tagTranslationHelp = {
		Num = L["The current number of combo points."]
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Combo points full",
	localName = L["Combo points full"],
	defaultTag = L["[Num] CP Finish It!"],
	tagTranslations = {
		Num = 1
	},
	tagTranslationHelp = {
		Num = L["The current number of combo points."]
	},
	color = "ff7f00", -- orange
}

onEnableFuncs[#onEnableFuncs+1] = function()
	mod:RegisterEvent("PLAYER_COMBO_POINTS", function()
		local num = GetComboPoints()
		if num == 0 then
			return
		end
		local info = new(num)
		Parrot:TriggerCombatEvent("Notification", num == 5 and "Combo points full" or "Combo point gain", info)
		info = del(info)
	end)
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Honor gains",
	localName = L["Honor gains"],
	defaultTag = "+[Amount] " .. HONOR_CONTRIBUTION_POINTS,
	parserEvent = {
		eventType = "Honor",
		amount_gt = 0,
		isHonorable = true,
	},
	tagTranslations = {
		Amount = "amount",
		Name = "sourceName",
		Rank = "sourceRank",
	},
	tagTranslationHelp = {
		Amount = L["The amount of honor gained."],
		Name = L["The name of the enemy slain."],
		Rank = L["The rank of the enemy slain."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation gains",
	localName = L["Reputation gains"],
	defaultTag = "+[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_gt = 0,
	},
	tagTranslations = {
		Amount = "amount",
		Faction = "faction",
	},
	tagTranslationHelp = {
		Amount = L["The amount of reputation gained."],
		Faction = L["The name of the faction."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation losses",
	localName = L["Reputation losses"],
	defaultTag = "-[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_lt = 0,
	},
	tagTranslations = {
		Amount = function(info) return -info.amount end,
		Faction = "faction",
	},
	tagTranslationHelp = {
		Amount = L["The amount of reputation lost."],
		Faction = L["The name of the faction."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Skill gains",
	localName = L["Skill gains"],
	defaultTag = "[Skill]: [Amount]",
	parserEvent = {
		eventType = "Skill",
	},
	tagTranslations = {
		Skill = retrieveAbilityName,
		Amount = "amount",
	},
	tagTranslationHelp = {
		Skill = L["The skill which experienced a gain."],
		Amount = L["The amount of skill points currently."]
	},
	color = "5555ff", -- semi-light blue
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Experience gains",
	localName = L["Experience gains"],
	defaultTag = "[Amount] " .. XP,
	parserEvent = {
		eventType = "Experience",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = "amount",
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Amount = L["The amount of experience points gained."]
	},
	color = "bf4ccc", -- magenta
	sticky = true,
	defaultDisabled = true,
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Player killing blows",
	localName = L["Player killing blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Skill = L["The spell or ability used to slay the enemy."],
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "NPC killing blows",
	localName = L["NPC killing blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Skill = L["The spell or ability used to slay the enemy."],
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

if playerClass == "WARLOCK" then
	Parrot:RegisterCombatEvent{
		category = "Notification",
		name = "Soul shard gains",
		localName = L["Soul shard gains"],
		defaultTag = "+[Name]",
		parserEvent = {
			eventType = "Create",
			itemString_match = "item:6265:[%-%d:]+",
			isCreated = true,
		},
		tagTranslations = {
			Name = "itemName",
			Icon = retrieveIconFromItemLink,
		},
		tagTranslationHelp = {
			Name = L["The name of the soul shard."],
		},
		color = "990099", -- purple
		sticky = true,
	}
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Extra attacks",
	localName = L["Extra attacks"],
	defaultTag = L["%s!"]:format("[Skill]"),
	parserEvent = {
		eventType = "Extra Attack",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationHelp = {
		Skill = L["The name of the spell or ability which provided the extra attacks."],
	},
	color = "ffff00", -- yellow
	sticky = true,
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Skill cooldown finish",
	localName = L["Skill cooldown finish"],
	defaultTag = L["[[Skill] ready!]"],
	tagTranslations = {
		Skill = 1,
		Icon = 2,
	},
	tagTranslationHelp = {
		Skill = L["The name of the spell or ability which is ready to be used."],
	},
	color = "ffffff", -- white
	sticky = false,
}

local cooldowns = {}
local spellNameToID = {}
local spellNameToTree = {}

local function resetCooldownState()
	local GCD = 1.5
	if spellNameToID[BS["Shoot"]] then
		local _, shootCooldown = GetSpellCooldown(spellNameToID[BS["Shoot"]], "spell")
		if shootCooldown > GCD then
			GCD = shootCooldown
		end
	end
	for name, id in pairs(spellNameToID) do
		local start, duration = GetSpellCooldown(id, "spell")
		cooldowns[name] = start > 0 and duration > GCD
	end
end
local function resetSpells()
	for k in pairs(spellNameToID) do
		spellNameToID[k] = nil
	end
	for k in pairs(cooldowns) do
		cooldowns[k] = nil
	end
	for i = 1, GetNumSpellTabs() do
		local _, _, offset, num = GetSpellTabInfo(i)
		for j = 1, num do
			local id = offset+j
			local spell = GetSpellName(id, "spell")
			spellNameToID[spell] = id
			spellNameToTree[spell] = j
		end
	end
	
	resetCooldownState()
end

local groups = {
	[BS["Freezing Trap"]] = L["Traps"],
	[BS["Frost Trap"]] = L["Traps"],
	[BS["Immolation Trap"]] = L["Traps"],
	[BS["Snake Trap"]] = L["Traps"],
	[BS["Explosive Trap"]] = L["Traps"],
	[BS["Frost Shock"]] = L["Shocks"],
	[BS["Flame Shock"]] = L["Shocks"],
	[BS["Earth Shock"]] = L["Shocks"],
}

local function cooldownOnUpdate()
	local GCD = 1.5
	if spellNameToID[BS["Shoot"]] then
		local _, shootCooldown = GetSpellCooldown(spellNameToID[BS["Shoot"]], "spell")
		if shootCooldown > GCD then
			GCD = shootCooldown
		end
	end
	local groupsToTrigger = new()
	local spellsToTrigger = new()
	local treeCount = new()
	for name, id in pairs(spellNameToID) do
		local old = cooldowns[name]
		local start, duration = GetSpellCooldown(id, "spell")
		local check = start > 0 and duration > GCD
		cooldowns[name] = check
		if not check and old then
			if groups[name] then
				groupsToTrigger[groups[name]] = true
			else
				spellsToTrigger[name] = id
				local tree = spellNameToTree[name]
				treeCount[tree] = (treeCount[tree] or 0) + 1
			end
		end
	end
	for name in pairs(groupsToTrigger) do
		local info = new(name)
		Parrot:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
		info = del(info)
	end
	groupsToTrigger = del(groupsToTrigger)
	for tree, num in pairs(treeCount) do
		if num >= 3 then
			for name in pairs(spellsToTrigger) do
				if tree == spellNameToTree[name] then
					spellsToTrigger[name] = nil
				end
			end
			local name, texture = GetSpellTabInfo(tree)
			local info = new(L["%s Tree"]:format(name), texture)
			Parrot:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
			info = del(info)
		end
	end
	treeCount = del(treeCount)
	for name, id in pairs(spellsToTrigger) do
		local info = new(name, GetSpellTexture(id, "spell"))
		Parrot:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
		info = del(info)
	end
	spellsToTrigger = del(spellsToTrigger)
end

onEnableFuncs[#onEnableFuncs+1] = function()
	resetSpells()
	
	mod:ScheduleRepeatingEvent("Parrot_CombatEvents_Data-cooldownOnUpdate", cooldownOnUpdate, 0.1)
	mod:RegisterEvent("SPELLS_CHANGED", resetSpells)
	mod:RegisterEvent("SPELL_UPDATE_COOLDOWN", resetCooldownState)
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Loot",
	localName = L["Loot"],
	defaultTag = L["Loot [Name] +[Amount]([Total])"],
	parserEvent = {
		eventType = "Create",
		sourceID = "player",
		itemName_not = false,
		isCreated = false,
		itemRarity_gt = 0,
	},
	tagTranslations = {
		Name = function(info)
			local name, _, rarity = GetItemInfo(info.itemLink or info.itemName)
			local color = ITEM_QUALITY_COLORS[rarity]
			if color then
				return color.hex .. name .. "|r"
			else
				return name
			end
		end,
		Amount = "amount",
		Total = function(info)
			local oldTotal = GetItemCount(info.itemLink or info.itemName)
			return oldTotal + info.amount
		end,
		Icon = retrieveIconFromItemLink,
	},
	tagTranslationHelp = {
		Name = L["The name of the item."],
		Amount = L["The amount of items looted."],
		Total = L["The total amount of items in inventory."],
	},
	color = "ffffff", -- white
}

local function utf8trunc(text, num)
	local len = 0
	local i = 1
	local text_len = #text
	while len < num and i <= text_len do
		len = len + 1
		local b = text:byte(i)
		if b <= 127 then
			i = i + 1
		elseif b <= 223 then
			i = i + 2
		elseif b <= 239 then
			i = i + 3
		else
			i = i + 4
		end
	end
	return text:sub(1, i-1)
end

local GOLD_ABBR = utf8trunc(_G.GOLD, 1)
if GOLD_ABBR:len() == 1 then
	GOLD_ABBR = GOLD_ABBR:lower()
end
local SILVER_ABBR = utf8trunc(_G.SILVER, 1)
if SILVER_ABBR:len() == 1 then
	SILVER_ABBR = SILVER_ABBR:lower()
end
local COPPER_ABBR = utf8trunc(_G.COPPER, 1)
if COPPER_ABBR:len() == 1 then
	COPPER_ABBR = COPPER_ABBR:lower()
end
Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Loot money",
	localName = L["Loot money"],
	defaultTag = L["Loot +[Amount]"],
	parserEvent = {
		eventType = "Create",
		sourceID = "player",
		itemName = false,
		isCreated = false,
	},
	tagTranslations = {
		Amount = function(info)
			local value = info.amount
			if value >= 10000 then
				return ("%d|cffffd700%s|r%d|cffc7c7cf%s|r%d|cffeda55f%s|r"):format(value/10000, GOLD_ABBR, (value/100)%100, SILVER_ABBR, value%100, COPPER_ABBR)
			elseif value >= 100 then
				return ("%d|cffc7c7cf%s|r%d|cffeda55f%s|r"):format(value/100, SILVER_ABBR, value%100, COPPER_ABBR)
			else
				return ("%d|cffeda55f%s|r"):format(value, COPPER_ABBR)
			end
		end,
--		Icon = function()
--			return ""
--		end
	},
	tagTranslationHelp = {
		Amount = L["The amount of gold looted."],
	},
	color = "ffffff", -- white
}
