local VERSION = tonumber(("$Revision: 44596 $"):match("%d+"))

local Parrot = Parrot
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-23 03:35:46 -0400 (Mon, 23 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local mod = Parrot:NewModule("TriggerConditions_Data", "AceEvent-2.0", "Parser-3.0")

local onEnableFuncs = {}
function mod:OnEnable()
	for _,v in ipairs(onEnableFuncs) do
		v()
	end
end

local L = Parrot:L("Parrot_TriggerConditions_Data")

Parrot:RegisterPrimaryTriggerCondition {
	name = "Enemy target health percent",
	localName = L["Enemy target health percent"],
	defaultParam = 0.5,
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("target") or not UnitCanAttack("player", "target") or UnitIsDeadOrGhost("target") then
			return nil
		else
		 	return UnitHealth("target")/UnitHealthMax("target")
		end
	end,
	events = {
		UNIT_HEALTH = "target",
		UNIT_MAXHEALTH = "target",
		UNIT_FACTION = "target",
		PLAYER_TARGET_CHANGED = true,
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Friendly target health percent",
	localName = L["Friendly target health percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("target") or not UnitIsFriend("player", "target") or UnitIsDeadOrGhost("target") then
			return nil
		else
			return UnitHealth("target")/UnitHealthMax("target")
		end
	end,
	events = {
		UNIT_HEALTH = "target",
		UNIT_MAXHEALTH = "target",
		UNIT_FACTION = "target",
		PLAYER_TARGET_CHANGED = true,
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Self health percent",
	localName = L["Self health percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if UnitIsDeadOrGhost("player") then
			return nil
		else
			return UnitHealth("player")/UnitHealthMax("player")
		end
	end,
	events = {
		UNIT_HEALTH = "player",
		UNIT_MAXHEALTH = "player",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Self mana percent",
	localName = L["Self mana percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if UnitIsDeadOrGhost("player") or UnitPowerType("player") ~= 0 then
			return nil
		else
			return UnitMana("player")/UnitManaMax("player")
		end
	end,
	events = {
		UNIT_MANA = "player",
		UNIT_MAXMANA = "player",
		UNIT_DISPLAYPOWER = "player",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Pet health percent",
	localName = L["Pet health percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("pet") or UnitIsDeadOrGhost("pet") then
			return nil
		end
		return UnitHealth("pet")/UnitHealthMax("pet")
	end,
	events = {
		UNIT_HEALTH = "pet",
		UNIT_MAXHEALTH = "pet",
		PLAYER_PET_CHANGED = "pet",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Incoming block",
	localName = L["Incoming block"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Incoming crit",
	localName = L["Incoming crit"],
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		isCrit = true,
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Incoming dodge",
	localName = L["Incoming dodge"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Incoming parry",
	localName = L["Incoming parry"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Outgoing block",
	localName = L["Outgoing block"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Outgoing crit",
	localName = L["Outgoing crit"],
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		isCrit = true,
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Outgoing dodge",
	localName = L["Outgoing dodge"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Outgoing parry",
	localName = L["Outgoing parry"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Outgoing cast",
	localName = L["Outgoing cast"],
	parserEvent = {
		eventType = "Cast",
		sourceID = "player",
		recipientID_not = "player",
		isBegin = false,
	},
	parserArg = "abilityName",
	param = {
		type = 'text',
		usage = L["<Skill name>"],
	},
}

onEnableFuncs[#onEnableFuncs+1] = function()
	mod:RegisterParserEvent({
		eventType_in = { "Damage", "Heal" },
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	}, function(info)
		Parrot:FirePrimaryTriggerCondition("Outgoing cast", info.abilityName)
	end)
end

Parrot:RegisterPrimaryTriggerCondition {
	name = "Incoming cast",
	localName = L["Incoming cast"],
	parserEvent = {
		eventType = "Cast",
		recipientID = "player",
		isBegin = false,
	},
	parserArg = "abilityName",
	param = {
		type = 'text',
		usage = L["<Skill name>"],
	},
}

onEnableFuncs[#onEnableFuncs+1] = function()
	mod:RegisterParserEvent({
		eventType_in = { "Damage", "Heal" },
		recipientID = "player",
		abilityName_not = false,
	}, function(info)
		Parrot:FirePrimaryTriggerCondition("Incoming cast", info.abilityName)
	end)
end

Parrot:RegisterPrimaryTriggerCondition {
	name = "Self buff gain",
	localName = L["Self buff gain"],
	events = {
		SpecialEvents_PlayerBuffGained = true
	},
	param = {
		type = 'text',
		usage = L["<Buff name>"],
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Self buff fade",
	localName = L["Self buff fade"],
	events = {
		SpecialEvents_PlayerBuffLost = true
	},
	param = {
		type = 'text',
		usage = L["<Buff name>"],
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Self debuff gain",
	localName = L["Self debuff gain"],
	events = {
		SpecialEvents_PlayerDebuffGained = true
	},
	param = {
		type = 'text',
		usage = L["<Debuff name>"],
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Self debuff fade",
	localName = L["Self debuff fade"],
	events = {
		SpecialEvents_PlayerDebuffLost = true
	},
	param = {
		type = 'text',
		usage = L["<Debuff name>"],
	},
}

Parrot:RegisterPrimaryTriggerCondition {
	name = "Target buff gain",
	localName = L["Target buff gain"],
	param = {
		type = 'text',
		usage = L["<Buff name>"],
	},
}

onEnableFuncs[#onEnableFuncs+1] = function()
	mod:RegisterParserEvent({
		eventType = "Aura",
		isBuff = true,
	}, function(info)
		if UnitExists("target") and info.recipientName == UnitName("target") then
			Parrot:FirePrimaryTriggerCondition("Target buff gain", info.abilityName)
		end
	end)
end

Parrot:RegisterPrimaryTriggerCondition {
	name = "Target debuff gain",
	localName = L["Target debuff gain"],
	param = {
		type = 'text',
		usage = L["<Debuff name>"],
	},
}

onEnableFuncs[#onEnableFuncs+1] = function()
	mod:RegisterParserEvent({
		eventType = "Aura",
		isBuff = false,
	}, function(info)
		if UnitExists("target") and info.recipientName == UnitName("target") then
			Parrot:FirePrimaryTriggerCondition("Target debuff gain", info.abilityName)
		end
	end)
end

Parrot:RegisterSecondaryTriggerCondition {
	name = "Buff inactive",
	localName = L["Buff inactive"],
	notLocalName = L["Buff active"],
	param = {
		type = 'text',
		usage = "<Buff name>",
	},
	check = function(param)
		return not SEAura:UnitHasBuff("player", param)
	end,
}

Parrot:RegisterSecondaryTriggerCondition {
	name = "Minimum power amount",
	localName = L["Minimum power amount"],
	defaultParam = 0.5,
	param = {
		type = 'range',
		min = 0,
		max = playerClass == "WARRIOR" and 100 or playerClass == "ROGUE" and 120 or 10000,
		step = 1,
		bigStep = 50,
	},
	check = function(param)
		if UnitIsDeadOrGhost("player") then
			return false
		end
		return UnitMana("player")/UnitManaMax("player") >= param
	end,
}

local spellIDs = setmetatable({}, {__index=function(self, key)
	local _, _, offset, num = GetSpellTabInfo(GetNumSpellTabs())
	local total = offset + num
	for i = 1, total do
		if key == GetSpellName(i, "spell") then
			self[key] = i
			return i
		end
	end
	self[key] = false
	return false
end})

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot:RegisterEvent("SPELLS_CHANGED", function()
		for k in pairs(spellIDs) do
			spellIDs[k] = nil
		end
	end)
	for k in pairs(spellIDs) do
		spellIDs[k] = nil
	end
end

Parrot:RegisterSecondaryTriggerCondition {
	name = "Spell ready",
	localName = L["Spell ready"],
	param = {
		type = 'text',
		usage = "<Spell name>",
	},
	check = function(param)
		local id = spellIDs[param]
		if id then
			local start, duration = GetSpellCooldown(id, "spell")
			return start <= 0 or duration <= 1.5
		end
	end,
}

Parrot:RegisterSecondaryTriggerCondition {
	name = "Warrior stance",
	localName = L["Warrior stance"],
	notLocalName = L["Not in warrior stance"],
	param = {
		type = 'text',
		validate = {
			["Battle Stance"] = L["Battle Stance"],
			["Defensive Stance"] = L["Defensive Stance"],
			["Berserker Stance"] = L["Berserker Stance"],
		}
	},
	check = function(param)
		if playerClass ~= "WARRIOR" then
			return true
		end
		local form = GetShapeshiftForm(true)
		if form == 1 then
			return param == "Battle Stance"
		elseif form == 2 then
			return param == "Defensive Stance"
		elseif form == 3 then
			return param == "Berserker Stance"
		end
		return false
	end,
}
